@@ -306,13 +306,77 @@ void jed_set_buffer_ctime (Buffer *b)
    
    strcpy (file, b->dir);
    strcat (file, b->file);
-
+#if 0
    if (0 == sys_file_mod_time (file))
      b->c_time = 0;
    else
      b->c_time = sys_time ();
+#else
+   b->c_time = sys_file_mod_time (file);
+#endif
 }
 
+#if (defined(__MSDOS__) || defined(__WIN32__)) && !defined(W_OK)
+# define W_OK 2
+# define F_OK 0
+#endif
+
+#ifdef __GO32__
+# define access i386_access
+#endif
+
+int jed_buffer_file_is_readonly (Buffer *b)
+{
+#ifndef W_OK
+   return 0;
+#else
+   char *file;
+   int ro = 0;
+
+   if ((b->file == NULL)
+       || (b->file[0] == 0))
+     return 0;
+
+   file = jed_dir_file_merge (b->dir, b->file);
+   if (file == NULL)
+     return -1;
+
+   if (0 == access(file, F_OK))
+     {
+	if (-1 == access(file, W_OK))
+	  ro = 1;
+     }
+   else
+     {
+	/* file does not exist.  Can we write to the directory? */
+	/* knock of slash since some primitive systems cannot handle the 
+	 * final slash in a path name. 
+	 */
+	char *dir;
+#ifdef IBMPC_SYSTEM
+	dir = extract_file (file);
+	*dir = 0;
+	if (strlen (file) > 3)   /* allow C:/, but not C:/xx/ */
+	  *--dir = 0;
+	dir = file;
+#else
+	dir = b->dir;
+#endif
+	if ((0 == access (dir, F_OK))
+#ifdef VMS
+	    && (-1 == access (dir, X_OK))
+#else
+	    && (-1 == access (dir, W_OK))
+#endif
+	    )
+	  ro = 1;
+     }
+   
+   SLfree (file);
+   return ro;
+#endif				       /* W_OK */
+}	     
+
 void set_file_modes (void) /*{{{*/
 {
    char *type;
@@ -320,57 +384,12 @@ void set_file_modes (void) /*{{{*/
    if (CBuf == NULL) return;
 
    jed_set_buffer_ctime (CBuf);
+   if (1 == jed_buffer_file_is_readonly (CBuf))
+     CBuf->flags |= READ_ONLY;
 
    if ((CBuf->file != NULL)
        && (CBuf->file[0] != 0))
      {
-#if (defined(__MSDOS__) || defined(__WIN32__)) && !defined(W_OK)
-# define W_OK 2
-# define F_OK 0
-#endif
-
-#ifdef W_OK
-# ifdef __GO32__
-#  define access i386_access
-# endif
-	char *file = jed_dir_file_merge (CBuf->dir, CBuf->file);
-	if (file != NULL)
-	  {
-	     if (0 == access(file, F_OK))
-	       {
-		  if (-1 == access(file, W_OK))
-		    CBuf->flags |= READ_ONLY;
-	       }
-	     else 
-	       {
-		  /* file does not exist.  Can we write to the directory? */
-		  /* knock of slash since some primitive systems cannot handle the 
-		   * final slash in a path name. 
-		   */
-		  char *dir;
-#ifdef IBMPC_SYSTEM
-		  dir = extract_file (file);
-		  *dir = 0;
-		  if (strlen (file) > 3)   /* allow C:/, but not C:/xx/ */
-		    *--dir = 0;
-		  dir = file;
-#else
-		  dir = CBuf->dir;
-#endif
-		  if ((0 == access (dir, F_OK))
-#ifdef VMS
-		      && (-1 == access (dir, X_OK))
-#else
-		      && (-1 == access (dir, W_OK))
-#endif
-		      )
-		    CBuf->flags |= READ_ONLY;
-	       }
-
-	     SLfree (file);
-	  }
-	     
-#endif				       /* W_OK */
 	CBuf->flags |= AUTO_SAVE_BUFFER;
 	CBuf->hits = 0;
 	type = file_type(CBuf->file);
