@@ -802,7 +802,7 @@ static SLang_Intrin_Fun_Type Jed_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC("quit_jed", quit_jed, VOID_TYPE, 0),
    MAKE_INTRINSIC_S("read_file", find_file_cmd, INT_TYPE),
    MAKE_INTRINSIC_4("read_with_completion", read_object_with_completion, VOID_TYPE, STRING_TYPE, STRING_TYPE, STRING_TYPE, INT_TYPE),
-   MAKE_INTRINSIC_I("set_abort_char", set_abort_char, VOID_TYPE),
+   MAKE_INTRINSIC_I("set_abort_char", jed_set_abort_char, VOID_TYPE),
    MAKE_INTRINSIC("suspend", sys_spawn_cmd, VOID_TYPE, 0),
    MAKE_INTRINSIC_I("ungetkey", ungetkey, VOID_TYPE),
    MAKE_INTRINSIC_S("buffer_keystring", do_buffer_keystring, VOID_TYPE),
@@ -878,6 +878,7 @@ static SLang_Intrin_Fun_Type Jed_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_I("send_process_eof", jed_send_process_eof, VOID_TYPE),
    MAKE_INTRINSIC_I("get_process_input", get_process_input, VOID_TYPE),
    MAKE_INTRINSIC_II("signal_process", jed_signal_process, VOID_TYPE),
+   MAKE_INTRINSIC_II("signal_fg_process", jed_signal_fg_process, VOID_TYPE),
    MAKE_INTRINSIC_II("process_query_at_exit", jed_query_process_at_exit, VOID_TYPE),
 #endif
 
@@ -977,10 +978,10 @@ static SLang_Intrin_Var_Type Jed_Variables [] =
    MAKE_VARIABLE("_jed_version_string", &Jed_Version_String, STRING_TYPE, 1),
    MAKE_VARIABLE("_jed_secure_mode", &Jed_Secure_Mode, INT_TYPE, 1),
 
+   MAKE_VARIABLE("Simulate_Graphic_Chars", &Jed_Simulate_Graphic_Chars, INT_TYPE, 0),
 #ifndef IBMPC_SYSTEM
    MAKE_VARIABLE("OUTPUT_RATE", &Obsolete_Int_Variable, INT_TYPE, 0),
    MAKE_VARIABLE("USE_ANSI_COLORS", &tt_Use_Ansi_Colors, INTP_TYPE, 0),
-   MAKE_VARIABLE("Simulate_Graphic_Chars", &Jed_Simulate_Graphic_Chars, INT_TYPE, 0),
    MAKE_VARIABLE("TERM_CANNOT_INSERT", &tt_Term_Cannot_Insert, INTP_TYPE, 0),
    MAKE_VARIABLE("TERM_CANNOT_SCROLL", &tt_Term_Cannot_Scroll, INTP_TYPE, 0),
    MAKE_VARIABLE("TERM_BLINK_MODE", &tt_Blink_Mode, INTP_TYPE, 0),
