@@ -45,6 +45,8 @@
 /*}}}*/
 
 void (*X_Define_Keys_Hook)(SLKeyMap_List_Type *);
+int (*X_Set_Abort_Char_Hook) (unsigned char);
+unsigned char Jed_Abort_Char = 7;		       /* ^G */
 
 typedef struct /*{{{*/
 {
@@ -86,6 +88,7 @@ static int redraw (void)
    return 0;
 }
 
+#if HAS_MOUSE
 #ifndef IBMPC_SYSTEM
 static int xterm_mouse_cmd (void)
 {
@@ -126,6 +129,7 @@ static int xterm_mouse_cmd (void)
    return 1;
 }
 #endif
+#endif				       /* HAS_MOUSE */
 
 SLKeymap_Function_Type Jed_Functions[] = /*{{{*/
 {
@@ -137,7 +141,7 @@ SLKeymap_Function_Type Jed_Functions[] = /*{{{*/
      {"beg_of_line", bol},
      {"center_line", center_line},
      {"copy_region", copy_to_pastebuffer},
-#if defined(__MSDOS__) && !defined(__WIN32__)
+#if (defined(__MSDOS__) && !defined(__WIN32__)) || (defined(__QNX__) && defined(__WATCOMC__))
      {"coreleft", show_memory},
 #endif
      {"delete_char_cmd", delete_char_cmd},
@@ -407,6 +411,25 @@ void set_current_kbd_command (char *s) /*{{{*/
 
 /*}}}*/
 
+static void update_jed_keybuffer (void) /*{{{*/
+{
+   Last_Key_Buffer_Len = (int) (Key_Bufferp - Key_Buffer);
+   if (Last_Key_Buffer_Len == 1)
+     {
+	*Jed_Key_Buffer = *Key_Buffer;
+	*(Jed_Key_Buffer + 1) = 0;
+     }
+   else
+     {
+        *Key_Bufferp = 0;
+	strcpy (Jed_Key_Buffer, Key_Buffer);
+     }
+   
+   Key_Bufferp = Key_Buffer;
+}
+
+/*}}}*/
+
 static int do_macro_string (char *str, int repeat)
 {
    static int macro_depth;
@@ -432,7 +455,7 @@ static int do_macro_string (char *str, int repeat)
 	while ((SLang_Error == 0)
 	       && (SLKeyBoard_Quit == 0)
 	       && (Read_This_Character != NULL))
-	  do_key();
+	  jed_do_key();
 	
 	if (Read_This_Character != NULL)
 	  {
@@ -496,27 +519,8 @@ static int key_interpret (SLang_Key_Type *key) /*{{{*/
 
 /*}}}*/
 
-static void update_jed_keybuffer (void) /*{{{*/
-{
-   Last_Key_Buffer_Len = (int) (Key_Bufferp - Key_Buffer);
-   if (Last_Key_Buffer_Len == 1)
-     {
-	*Jed_Key_Buffer = *Key_Buffer;
-	*(Jed_Key_Buffer + 1) = 0;
-     }
-   else
-     {
-        *Key_Bufferp = 0;
-	strcpy (Jed_Key_Buffer, Key_Buffer);
-     }
-   
-   Key_Bufferp = Key_Buffer;
-}
 
-/*}}}*/
-
-
-int do_key (void) /*{{{*/
+static int do_key (void) /*{{{*/
 {
    SLang_Key_Type *key;
    int repeat;
@@ -567,6 +571,20 @@ int do_key (void) /*{{{*/
 
 /*}}}*/
 
+int jed_do_key (void)
+{
+   int ret = do_key ();
+   if (SLang_Error)
+     {
+	jed_abort_keyboard_macro ();
+	Read_This_Character = NULL;
+	Repeat_Factor = NULL;
+     }
+   return ret;
+}
+
+
+
 void do_jed(void) /*{{{*/
 {
    char *mode;
@@ -580,7 +598,7 @@ void do_jed(void) /*{{{*/
 
    Repeat_Factor = NULL;
    Replace_Preserve_Case = 0;
-   if (do_key()) JWindow->trashed = 1;
+   if (jed_do_key()) JWindow->trashed = 1;
    
     /* internal editing commands may have selected a different buffer
      * so put it back to the one associated with the window.
@@ -606,7 +624,6 @@ void jed(void) /*{{{*/
 {
    /* This routine is called from main.  So before actually strting, check
       one more hook to just to make sure  all things are go. */
-   char *startup = ".()jed_startup_hook";
    char *buffer_name;
 
 #if JED_HAS_TTY_MENUS
@@ -616,6 +633,10 @@ void jed(void) /*{{{*/
    (void) jed_va_run_hooks ("_jed_switch_active_buffer_hooks", JED_HOOKS_RUN_ALL, 1, "");
    /* This should not fail, and if it does it is not serious */
    buffer_name = SLang_create_slstring (CBuf->name);
+#if 1
+   (void) jed_va_run_hooks ("_jed_startup_hooks", JED_HOOKS_RUN_ALL, 0);
+#else
+   char *startup = ".()jed_startup_hook";
 
    if (2 == SLang_is_defined(startup + 3))
      {
@@ -623,7 +644,7 @@ void jed(void) /*{{{*/
 	/* remove its definition from memory */
 	SLang_load_string (startup);
      }
-   
+#endif
    
    Jump_Buffer_Ptr = &Jump_Buffer;
    
@@ -691,7 +712,7 @@ int digit_arg(void) /*{{{*/
    i = 0;
    buf[i++] = (char) SLang_Last_Key_Char;
    
-   /* After do_key (what called this), Key_Bufferp is reset.  However, I want 
+   /* After jed_do_key (what called this), Key_Bufferp is reset.  However, I want 
     * to keep it for echoing subsequent characters.  I restore its previous 
     * value so that echoing will continue. 
     */
@@ -719,7 +740,7 @@ int digit_arg(void) /*{{{*/
    if (Key_Bufferp != Key_Buffer) Key_Bufferp--;
    
    /* Key_Timeout is still active and is only reset after this call. */
-   do_key();
+   jed_do_key();
    return(1);
 }
 
@@ -902,25 +923,18 @@ char *what_keymap() /*{{{*/
 
 /*}}}*/
 
-void set_abort_char(int *c) /*{{{*/
+void jed_set_abort_char (int *c) /*{{{*/
 {
    char str[2];
    int i;
-#ifdef __MSDOS_16BIT__
-   char ch, *s, *scan = "@#$%^&*()_+?IQWERTYUIOP[]!!ASDFGHJKL!!!!\\ZXCVBNM";
-   
-   ch = 64 + (char) *c;
-   s = scan; while (*s && (*s != ch)) s++;
-   if (*s == 0) return;
-   Abort_Char = (int) (s - scan) + 0x03;
-#else
-   Abort_Char = *c;
-#endif
-#ifndef MSWINDOWS
-   reset_tty();
-   init_tty();
-#endif
-   str[0] = *c; str[1] = 0;
+   char ch = (char) *c;
+
+   if (X_Set_Abort_Char_Hook != NULL)
+     (void) (*X_Set_Abort_Char_Hook) ((unsigned char) ch);
+
+   Jed_Abort_Char = (int) ch;
+
+   str[0] = ch; str[1] = 0;
    
    for (i = 0; i < SLANG_MAX_KEYMAPS; i++)
      {
