@@ -1297,7 +1297,7 @@ int num_windows() /*{{{*/
 void flush_message(char *m) /*{{{*/
 {
    message(m);
-   if (Batch) return;
+   if (Batch || (JWindow == NULL)) return;
    do_dialog(Message_Buffer);
    SLsmg_gotorc (Jed_Num_Screen_Rows - 1, strlen(Message_Buffer));
    *Message_Buffer = 0;
@@ -2193,6 +2193,34 @@ void jed_setup_minibuffer_keymap (void)
 }
 
 
+#if SLANG_VERSION < 10405
+/* Work around a slang bug triggered by jed. Apps which use slang's 
+ * load_file function do not have this problem.
+ */
+extern int (*_SLprep_eval_hook) (char *);
+static int bug_fixed_prep_eval_expr (char *expr)
+{
+   int ret;
+   char *end;
+
+   end = strchr (expr, '\n');
+   if (end == NULL)
+     end = expr + strlen (expr);
+   expr = SLmake_nstring (expr, (unsigned int) (end - expr));
+   if (expr == NULL)
+     return -1;
+	
+   if ((0 != SLang_load_string (expr))
+       || (-1 == SLang_pop_integer (&ret)))
+     ret = -1;
+   else
+     ret = (ret != 0);
+
+   SLfree (expr);
+   return ret;
+}
+#endif
+
 void init_minibuffer() /*{{{*/
 {
    Buffer *tmp;
@@ -2219,6 +2247,10 @@ void init_minibuffer() /*{{{*/
    SLang_Enter_Function = enter_function;
    SLang_Exit_Function = exit_function;
 #endif
+#if SLANG_VERSION < 10405
+   _SLprep_eval_hook = bug_fixed_prep_eval_expr;
+#endif
+
    if ((-1 == SLang_init_slang ())
 #ifndef SIXTEEN_BIT_SYSTEM
        || (-1 == SLang_init_slmath ())
