@@ -911,7 +911,9 @@ static int copy_menu (Menu_Popup_Type *dest, Menu_Node_Type *src)
  *  SLsmg MenuBar interface
  */
 
-#ifndef IBMPC_SYSTEM
+#define USE_SIMULATED_LINES
+
+#ifdef USE_SIMULATED_LINES
 static void simulate_hline (unsigned int n)
 {
    while (n--)
@@ -944,7 +946,8 @@ static void simulate_box (int r, int c, unsigned int dr, unsigned int dc)
 	SLsmg_write_nchars ("|", 1);
      }
 }
-#endif
+#endif				       /* USE_SIMULATED_LINES */
+
 static void draw_name (char *name, int color0, int color1, unsigned int field_width)
 {
    char *s;
@@ -1179,7 +1182,7 @@ static int draw_popup_menu (Menu_Popup_Type *p, int r, int c)
    SLsmg_set_color_in_region (JMENU_SHADOW_COLOR, r + 1, c + 1, dr, dc);
    SLsmg_set_color (JMENU_POPUP_COLOR);
    /* SLsmg_fill_region (r, c, dr, dc, ' '); */
-#ifndef IBMPC_SYSTEM
+#ifdef USE_SIMULATED_LINES
    if (Jed_Simulate_Graphic_Chars)
      simulate_box (r, c, dr, dc);
    else
@@ -1239,7 +1242,7 @@ static int draw_popup_menu (Menu_Popup_Type *p, int r, int c)
 	  {
 	   case MENU_NODE_SEPARATOR:
 	     SLsmg_write_nchars (" ", 1);
-#ifndef IBMPC_SYSTEM
+#ifdef USE_SIMULATED_LINES
 	     if (Jed_Simulate_Graphic_Chars)
 	       simulate_hline (dc - 1);
 	     else
@@ -1666,7 +1669,9 @@ static SLKeymap_Function_Type Menu_Keymap_Functions [] =
 };
 
 #ifndef IBMPC_SYSTEM
+# ifdef HAS_MOUSE
 static int xterm_mouse_cmd (void);
+# endif
 #endif
 
 static int init_menu_keymap (void)
@@ -1720,7 +1725,9 @@ static int init_menu_keymap (void)
    SLkm_define_key ("^(kP)", (FVOID_STAR) pgup_menu_cmd, Menu_Keymap);
    SLkm_define_key ("^(kN)", (FVOID_STAR) pgdn_menu_cmd, Menu_Keymap);
 # endif
+# ifdef HAS_MOUSE
    SLkm_define_key ("\033[M", (FVOID_STAR) xterm_mouse_cmd, Menu_Keymap);
+# endif
 #else				       /* IBMPC_SYSTEM */
 #include "doskeys.h"
    
@@ -2324,6 +2331,7 @@ int jed_init_menus (void)
    return 0;
 }
 
+#ifdef HAS_MOUSE
 static int select_menu_via_rc (int type, int r, int c)
 {
    Menu_Popup_Type *p;
@@ -2352,7 +2360,7 @@ static int select_menu_via_rc (int type, int r, int c)
 	  {
 	     if (type != JMOUSE_DRAG)
 	       return -1;
-	     
+
 	     if (Active_Popup->type != MENU_NODE_MENUBAR)
 	       return 0;
 	  }
@@ -2411,7 +2419,6 @@ static int select_menu_via_rc (int type, int r, int c)
    return 0;
 }
 
-
 int jed_menu_handle_mouse (unsigned int type, 
 			   int x, int y, int button, int shift)
 {
@@ -2451,6 +2458,6 @@ static int xterm_mouse_cmd (void)
 }
 
 
-#endif
-
+#endif 				       /* !IBMPC_SYSTEM */
+#endif				       /* HAS_MOUSE */
 #endif				       /* JED_HAS_TTY_MENUS */
