@@ -43,17 +43,23 @@
 # include "jprocess.h"
 #endif
 
+#ifdef __WIN32__
+# define HAS_WHEEL_MOUSE_SUPPORT	0   /* needs fixed to use push_mouse_event */
+#else
+# define HAS_WHEEL_MOUSE_SUPPORT	0
+#endif
+
 #define MSW_STRING_TYPE   1
 #define MSW_INT_TYPE      2
 #define MSW_COLOR_TYPE    3
 
+#define MAX_STRING_SIZE	  128
 #define MAX_KEYS          128
 
 #define KEY_SHIFT         1
 #define KEY_CONTROL       2
 #define KEY_ALT	 	  4
 
-int Abort_Char = 7;
 int NumLock_Is_Gold = 0;
 int PC_Alt_Char = 27;
 int PC_Fn_Char = 0;
@@ -74,6 +80,50 @@ HINSTANCE _hPrev;
 extern HINSTANCE _hPrev;
 #endif
 
+#if HAS_WHEEL_MOUSE_SUPPORT
+
+#ifndef WM_MOUSEWHEEL
+# define WM_MOUSEWHEEL (WM_MOUSELAST+1)  /* message that will be supported by the OS */
+#endif
+
+/* Details of the mystery mouse wheel window */
+
+#define MOUSEZ_CLASSNAME  "MouseZ"            /* wheel window class */
+#define MOUSEZ_TITLE      "Magellan MSWHEEL"  /* wheel window title */
+
+#define MSH_WHEELMODULE_CLASS (MOUSEZ_CLASSNAME)
+#define MSH_WHEELMODULE_TITLE (MOUSEZ_TITLE)
+#define MSH_MOUSEWHEEL     "MSWHEEL_ROLLMSG"
+#define MSH_WHEELSUPPORT   "MSH_WHEELSUPPORT_MSG"  /* name of msg to send to query for wheel support */
+#define MSH_SCROLL_LINES   "MSH_SCROLL_LINES_MSG"
+
+/* Details of the mouse wheel functionality - not all currently used by Wjed */
+
+typedef struct
+{
+   HWND hwnd;                 
+   UINT uiMshMsgMouseWheel,   
+        uiMshMsg3DSupport,    
+        uiMshMsgScrollLines;  
+   INT  iScrollLines;
+   BOOL fActive;
+}
+MS_Wheel_Type;
+
+static MS_Wheel_Type Wheel_Mouse;
+
+/* Key prefix for a mouse wheel movement                            */
+/* and postfix character for up/down                                */
+/* i.e. wheel up generates MOUSEKEY_PREFIX MOUSEKEY_UP key sequence */
+/*      and wheel down generates MOUSEKEY_PREFIX MOUSEKEY_DOWN      */
+
+#define MOUSEKEY_PREFIX "\033\000"
+#define MOUSEKEY_UP     '\xff'
+#define MOUSEKEY_DOWN   '\xfe'
+
+#endif                                 /* HAS_WHEEL_MOUSE_SUPPORT */
+
+
 typedef struct
 {
    char  *name;
@@ -100,12 +150,12 @@ Color_Value_Type;
 
 typedef struct
 {
-   char        title[30];
+   char        title[MAX_STRING_SIZE];
    HWND        w;
    HDC         hdc;
    int         ndc;
 
-   char        font_name[50];
+   char        font_name[MAX_STRING_SIZE];
    int         font_height;
    int         font_bold;
    int         font_width;
@@ -193,7 +243,7 @@ static LPLOGPALETTE The_Lplgpl = NULL;
 static HPALETTE The_Hpalette = NULL;
 #endif
 
-Msw_Ini_Type Msw_Ini_List[] =
+static Msw_Ini_Type Msw_Ini_List[] =
 {
      { "Font",         MSW_STRING_TYPE,   "fixed",         This_Window.font_name },
      { "FontHeight",   MSW_INT_TYPE,      "12",            &This_Window.font_height },
@@ -454,14 +504,15 @@ static COLORREF msw_get_color(char *s, char *dflt)
 
 static void msw_get_default(Msw_Ini_Type *t)
 {
-   char s[50];
+   char s[MAX_STRING_SIZE];
 
    GetProfileString(szJedSection, t->name, t->dflt, s, sizeof(s));
 
    switch (t->type)
      {
       case MSW_STRING_TYPE:
-	strcpy((char *)t->buf, s);
+	strncpy((char *)t->buf, s, MAX_STRING_SIZE);
+	((char *)t->buf)[MAX_STRING_SIZE-1] = 0;
 	break;
 
       case MSW_INT_TYPE:
@@ -478,7 +529,7 @@ static void msw_get_default(Msw_Ini_Type *t)
 static void msw_get_defaults(void)
 {
    int i = 0;
-   char buf[20];
+   char buf[128];
 
    /* Check for standard color names in INI file.
     * Add they if cannot find. */
@@ -1136,7 +1187,7 @@ static void msw_get_terminfo (void)
 static void msw_set_color (int i, char *what, char *fg, char *bg)
 {
    int r, g, b;
-   char buf[30];
+   char buf[64];
 
    if ((i < 0) || (i > JMAX_COLORS))
      return;
@@ -1271,30 +1322,8 @@ static void push_mouse_event(int button, int x, int y, int state, int type)
 	type = JMOUSE_IGNORE_EVENT;
      }
 #endif
-
-   switch (button)
-     {
-      case WM_LBUTTONUP:
-      case WM_LBUTTONDOWN:
-	if (!(GetKeyState(VK_MENU) & 0x8000))
-	  {
-	     jm.button = JMOUSE_BUTTON_1;
-	     break;
-	  }
-	/* drop */
-      case WM_MBUTTONUP:
-      case WM_MBUTTONDOWN:
-	jm.button = JMOUSE_BUTTON_2;
-	break;
-
-      case WM_RBUTTONUP:
-      case WM_RBUTTONDOWN:
-	jm.button = JMOUSE_BUTTON_3;
-	break;
-
-      default:
-	return;
-     }
+   
+   jm.button = button;
 
    if ((type == JMOUSE_UP)
        && (0 == (jm.state & jm.button)))
@@ -1324,6 +1353,35 @@ static void push_mouse_event(int button, int x, int y, int state, int type)
    _putkey(id);
 }
 
+static void push_wm_mouse_event (int button, int x, int y, int state, int type)
+{
+   switch (button)
+     {
+      case WM_LBUTTONUP:
+      case WM_LBUTTONDOWN:
+	if (!(GetKeyState(VK_MENU) & 0x8000))
+	  {
+	     button = JMOUSE_BUTTON_1;
+	     break;
+	  }
+	/* drop */
+      case WM_MBUTTONUP:
+      case WM_MBUTTONDOWN:
+	button = JMOUSE_BUTTON_2;
+	break;
+
+      case WM_RBUTTONUP:
+      case WM_RBUTTONDOWN:
+	button = JMOUSE_BUTTON_3;
+	break;
+
+      default:
+	return;
+     }
+   
+   push_mouse_event (button, x, y, state, type);
+}
+
 static void msw_define_color(char *color_name, int *pr, int *pg, int *pb)
 {
    char buf[30];
@@ -1387,7 +1445,7 @@ unsigned char sys_getkey(void)
      {
 	SLKeyBoard_Quit = 0;
 	flush_input ();
-	return Abort_Char;
+	return Jed_Abort_Char;
      }
 
    n = my_getkey();
@@ -1466,7 +1524,7 @@ unsigned char sys_getkey(void)
      {
 	SLKeyBoard_Quit = 0;
 	flush_input ();
-	return (unsigned char)Abort_Char;
+	return Jed_Abort_Char;
      }
 
    return my_getkey();
@@ -1591,8 +1649,8 @@ static LRESULT CALLBACK process_key_down (HWND hWnd, UINT msg, WPARAM wParam, LP
       case 0x051:            /* KEYPAD PGDN */
       case 0x053:            /* KEYPAD DEL */
       case 0x052:            /* KEYPAD INSERT */
-	if (GetKeyState(VK_NUMLOCK) & 0x0001)
-	  return DefWindowProc(hWnd, msg, wParam, lParam);
+	if (!NumLock_Is_Gold && (GetKeyState(VK_NUMLOCK) & 0x0001))
+          return DefWindowProc(hWnd, msg, wParam, lParam);
 
  	state = 0;
  	if (key_state & KEY_SHIFT) state = 1;
@@ -1759,7 +1817,7 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
       case WM_CHAR:
 	if (Ignore_Wchar_Message == 0)
 	  {
-	     if ((int) wParam == Abort_Char)
+	     if ((int) wParam == (int) Jed_Abort_Char)
 	       {
 		  if (Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
 		  SLKeyBoard_Quit = 1;
@@ -1799,7 +1857,7 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 
 	if (!mouse_button_down) break;
 
-	push_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_UP);
+	push_wm_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_UP);
 	mouse_button_down--;
 	break;
 
@@ -1807,7 +1865,7 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
       case WM_MBUTTONDOWN:
       case WM_RBUTTONDOWN:
 	mouse_button_down++;
-	push_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_DOWN);
+	push_wm_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_DOWN);
 	break;
 
       case WM_MOUSEMOVE:
@@ -1878,7 +1936,38 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 	return 0;
 
       default:
-	return DefWindowProc(hWnd, msg, wParam, lParam);
+
+#if HAS_WHEEL_MOUSE_SUPPORT
+      /* Check for a mouse wheel movement message
+       * and translate it into a fake keystroke. This has now been
+       * tested on Win95,98,NT4,2000
+       */
+        if ((Wheel_Mouse.fActive 
+             && (msg == Wheel_Mouse.uiMshMsgMouseWheel))
+            || (msg == WM_MOUSEWHEEL))
+          {
+#if 0
+             _putkey(MOUSEKEY_PREFIX);
+             _putkey((int)wParam < 0 ? MOUSEKEY_DOWN : MOUSEKEY_UP);
+             return 0;
+#else
+	     /* FIXME!!! I would like an X-Windows compatible implementation
+	      * here.  Under X, the wheel mouse produces JMOUSE_BUTTON_4 and
+	      * JMOUSE_BUTTON_5 events.  jed/lib/mouse.sl maps JMOUSE_BUTTON_4
+	      * events to upward movement and JMOUSE_BUTTON_5 to downward 
+	      * movement in the buffer.
+	      */
+
+	     /* if ((int) wParam < 0) */
+	     /*   push_mouse_event (JMOUSE_BUTTON_4, x, y, state, JMOUSE_DOWN); */
+	     /* else */
+	     /*   push_mouse_event (JMOUSE_BUTTON_4, x, y, state, JMOUSE_DOWN); */
+	       
+	     return 0;
+#endif
+          }
+#endif
+        return DefWindowProc(hWnd, msg, wParam, lParam);
      }
 
    return 0;
@@ -2405,6 +2494,41 @@ int jed_init_w32_support (void)
    return w32_init_subprocess_support (1);
 }
 
+#if HAS_WHEEL_MOUSE_SUPPORT
+
+/* Initialise mouse wheel handling functionality */
+
+static int init_wheel_mouse (void)
+{
+   /* Find the mystery mouse window */
+   
+   Wheel_Mouse.hwnd = FindWindow(MSH_WHEELMODULE_CLASS, MSH_WHEELMODULE_TITLE);
+
+   /* Register messages to determine mousey information */
+   
+   Wheel_Mouse.uiMshMsgMouseWheel  = RegisterWindowMessage(MSH_MOUSEWHEEL);
+   Wheel_Mouse.uiMshMsg3DSupport   = RegisterWindowMessage(MSH_WHEELSUPPORT);
+   Wheel_Mouse.uiMshMsgScrollLines = RegisterWindowMessage(MSH_SCROLL_LINES);
+
+   /* If we have a wheel enquiry message, ask about the presence of a wheel */
+   
+   if (Wheel_Mouse.uiMshMsg3DSupport)
+     Wheel_Mouse.fActive = (BOOL)SendMessage(Wheel_Mouse.hwnd, Wheel_Mouse.uiMshMsg3DSupport, 0, 0);
+   else
+     Wheel_Mouse.fActive = FALSE;  
+
+   /* If we have a scroll line enquiry message ask about that */
+   
+   if (Wheel_Mouse.uiMshMsgScrollLines)
+     Wheel_Mouse.iScrollLines = (int)SendMessage(Wheel_Mouse.hwnd, Wheel_Mouse.uiMshMsgScrollLines, 0, 0);
+   else
+     Wheel_Mouse.iScrollLines = 3;
+   
+   return 0;
+}
+#endif                                 /* HAS_WHEEL_MOUSE_SUPPORT */
+
+
 #if defined(__BORLANDC__) || defined(__WIN32__) || defined(__VC__)
 extern int main(int, char **);
 
@@ -2429,6 +2553,10 @@ int PASCAL WinMain(HINSTANCE inst, HINSTANCE pinst, LPSTR lpszCmdLine, int nCmdS
 #endif
    (void) nCmdShow;
 
+#if HAS_WHEEL_MOUSE_SUPPORT
+   (void) init_wheel_mouse ();
+#endif
+
    strcpy(command_line, "wjed ");
    strcat(command_line, lpszCmdLine);
    _hPrev = pinst;
